#include "../include/bic.h"

/*********************************************************
BIC
*********************************************************/

int *Quantize_colors(CImage *img, int color_dim){
  unsigned long i;
  unsigned long r, g, b;
  unsigned long fator_g, fator_b;
  int *color, n;
  
  n = img->C[0]->nrows * img->C[0]->ncols;  

  color = (int *) calloc(n, sizeof(int));
  if(color==NULL){
    printf("\nOut of memory \n");
    exit(-1);
  }
  
  fator_g = color_dim;
  fator_b = fator_g*color_dim;
  
  for(i=0; i<n; i++){
    r = color_dim*img->C[0]->val[i]/256;
    g = color_dim*img->C[1]->val[i]/256;
    b = color_dim*img->C[2]->val[i]/256;
    
    color[i] = (r + fator_g*g + fator_b*b);
  }
  return color;
}
/***************************************************************************/
unsigned char Compute_log(float value){
  unsigned char result;
  
  value = 255. * value;
  if(value==0.)       result=0;
  else if(value<1.)   result=1;
  else if(value<2.)   result=2;
  else if(value<4.)   result=3;
  else if(value<8.)   result=4;
  else if(value<16.)  result=5;
  else if(value<32.)  result=6;
  else if(value<64.)  result=7;
  else if(value<128.) result=8;
  else                result=9;
  
  return result;
}

/**************************************************************************/
void Compress_histogram(unsigned char *ch, unsigned long *h, 
      unsigned long max, int size){
  int i;
  unsigned char v;
  
  for(i=0; i<size; i++){
    v = Compute_log((float) h[i] / (float) max);
    ch[i] = (unsigned char)(48 + v);
  }
}


/***************************************************************************
  Este é o algoritmo BIC para classificar a cor entre interior/borda.
****************************************************************************/
void Compute_frequency_property(Image *img, Property *ppt){
  
  unsigned long x, y, p, q;
  int i, border;
  AdjRel *A;
  Pixel v;
  
  A = Circular(1.0);
  
  for(y=0L; y<img->nrows; y++){
    for(x=0L; x<img->ncols; x++){
      p = x + img->tbrow[y];
      border=FALSE;
      for (i=1; i < A->n; i++){
        v.x = x + A->dx[i];
        v.y = y + A->dy[i];
        if (ValidPixel(img,v.x,v.y)){
          q = v.x + img->tbrow[v.y];
          if(ppt[p].color!=ppt[q].color){ 
            border=TRUE;
            break;
          }
        }
      }
      if(border==FALSE) ppt[p].frequency=LOW;
      else ppt[p].frequency=HIGH;
    }
  }
  DestroyAdjRel(&A);
}

/****************************************************************************/
Property *Compute_pixels_properties(CImage *img)
{ 
  Property *p;
  int *color, i, n;
  
  n = img->C[0]->nrows * img->C[0]->ncols;  
  
  p = (Property *) calloc(n, sizeof(Property));
  if(p==NULL){
    printf("\nOut of memory \n");
    exit(-1);
  }
  
  color = Quantize_colors(img, 4);
  for(i=0; i<n; i++) 
    p[i].color=color[i];
  Compute_frequency_property(img->C[0], p);
  
  free(color);
  return p;
}
/****************************************************************************/
VisualFeature *Compute_histograms(Property *p, int n){
  VisualFeature *vf = (VisualFeature *) calloc(1,sizeof(VisualFeature));
  unsigned long i;
  
  for(i=0; i<DESC_SIZE; i++){
    vf->colorH[i] = 0;
    vf->lowH[i] = 0;
    vf->highH[i] = 0;
  }
  
  for(i=0; i<n; i++){
    /*GCH*/
    vf->colorH[p[i].color]++;
    
    /*Frequency*/
    if(p[i].frequency==LOW) 
      vf->lowH[p[i].color]++;
    else 
      vf->highH[p[i].color]++;
  }
  return vf;
}

/**************************************************************************/

CompressedVisualFeature *Compress_histograms(VisualFeature *vf, int npixels)
{
  CompressedVisualFeature *cvf =  (CompressedVisualFeature *) calloc(1,sizeof(CompressedVisualFeature));
  
  Compress_histogram(cvf->colorH, vf->colorH, npixels, DESC_SIZE);
  Compress_histogram(cvf->lowH, vf->lowH, npixels, DESC_SIZE);
  Compress_histogram(cvf->highH, vf->highH, npixels, DESC_SIZE);
  
  return cvf;
}

/*************************************************************************/
Curve *BIC(CImage *img)
{
  Property *p;
  VisualFeature *vf;
  CompressedVisualFeature *cvf;
  int i, npixels;
  Curve *curve = CreateCurve(2*DESC_SIZE);
  
  npixels = img->C[0]->nrows * img->C[0]->ncols;
  
  p = Compute_pixels_properties(img);
  //calcula o canny do img
  //pega frequency do vetor p e muda de acordo com o valor do Canny do pixel
  //Daqui pra baixo nao muda nada
  vf = Compute_histograms(p, npixels);
  cvf = Compress_histograms(vf, npixels);
  
  for (i=0; i<DESC_SIZE; i++){
    curve->X[i] = i;
    curve->Y[i] = cvf->lowH[i];
    curve->X[i+DESC_SIZE] = i+DESC_SIZE;
    curve->Y[i+DESC_SIZE] = cvf->highH[i];
  }

  free(p);
  free(vf);
  free(cvf);
  return curve;
}

/***************************************************************************/

void Write_visual_features(char *filename,char *dbname, CompressedVisualFeature *cvf)
{
  FILE *file;
  int i;
  
  if((file = fopen(dbname, "a+t")) == NULL)
    {
      fprintf(stderr, "Can't open %s \n", dbname);
      exit(-1);
    }
  
  fprintf(file, "%s\t", filename);
  for(i=0;i<DESC_SIZE;i++)
    {
      fprintf(file, "%c%c", cvf->lowH[i], cvf->highH[i]);
      /*fprintf(file, "%c%c%c", cvf->colorH[i], cvf->lowH[i], cvf->highH[i]);*/
       }
  fprintf(file, "\n");
  
  fclose(file);
}

/**************************************************************************/
CompressedVisualFeature *Extract_visual_features(CImage *img)
{
  Property *p;
  VisualFeature *vf;
  CompressedVisualFeature *cvf;
  int npixels;
  
  npixels = img->C[0]->nrows * img->C[0]->ncols;
  
  p = Compute_pixels_properties(img);
  vf = Compute_histograms(p, npixels);
  cvf = Compress_histograms(vf, npixels);
  
  free(p);
  return cvf;
}
/******************************************************************/
double gray_level_BIC(Image *img1, Image *img2){
  Property *p1, *p2;
  VisualFeature *vf1, *vf2;
  CompressedVisualFeature *cvf1, *cvf2;
  int i, n1, n2;
  double dist;

  /* computing properties*/
  n1 = img1->nrows * img1->ncols;  
  n2 = img2->nrows * img2->ncols;  

  p1 = (Property *) calloc(n1, sizeof(Property));
  p2 = (Property *) calloc(n2, sizeof(Property));

  if((p1==NULL)||(p2==NULL)){
    printf("\nOut of memory \n");
    exit(-1);
  }
  
  for(i=0; i<n1; i++){
    p1[i].color=img1->val[i];
  }
  for(i=0; i<n2; i++){
    p2[i].color=img2->val[i];
  }
  
  Compute_frequency_property(img1,p1);
  Compute_frequency_property(img2,p2);
  /***********************************/
  
  vf1 = Compute_histograms(p1, n1);
  vf2 = Compute_histograms(p2, n2);
  
  cvf1 = Compress_histograms(vf1, n1);
  cvf2 = Compress_histograms(vf2, n2);
  
  dist = 0.0;
  for (i=0; i<DESC_SIZE; i++){
    dist += fabs(cvf1->lowH[i] - cvf2->lowH[i]);
    dist += fabs(cvf1->highH[i] - cvf2->highH[i]);
  }
  
  free(p1);
  free(p2);
  free(vf1);
  free(vf2);
  free(cvf1);
  free(cvf2);
  return dist;
}