#include "../include/curve.h"
#include "../include/common.h"

Curve *CreateCurve(int n)
{
  Curve *curve=NULL;

  curve = (Curve *) calloc(1,sizeof(Curve));
  if (curve != NULL) {
    curve->X = AllocDoubleArray(n);
    curve->Y = AllocDoubleArray(n);
    curve->n = n;
  } else {
    Error(MSG1,"CreateCurve");
  }
  return(curve);
}

void DestroyCurve(Curve **curve)
{
  Curve *aux;

  aux = *curve;
  if (aux != NULL){
    if (aux->X != NULL) free(aux->X);
    if (aux->Y != NULL) free(aux->Y);
    free(aux);
    *curve = NULL;
  }
}

Curve *ReadCurve(char *filename)
{
  FILE *fp;
  int i,n;
  Curve *curve;
  float x,y;
  
  fp = fopen(filename,"r");
  if (fp == NULL){
    fprintf(stderr,"Cannot open %s\n",filename);
    exit(-1);
  }
  if(fscanf(fp,"%d\n",&n) >= 0);
  curve= CreateCurve(n);
  for (i=0; i < curve->n; i++) {
    if(fscanf(fp,"%f %f\n",&x,&y) >= 0);
    curve->X[i] = (double)x;
    curve->Y[i] = (double)y;
  }
  fclose(fp);
  return (curve);
}

void WriteCurve(Curve *curve,char *filename)
{
  FILE *fp;
  int i;
  
  fp = fopen(filename,"w");
  if (fp == NULL){
    fprintf(stderr,"Cannot open %s\n",filename);
    exit(-1);
  }
  fprintf(fp,"%d\n",curve->n);
  for (i=0; i < curve->n; i++)
    fprintf(fp,"%f %f\n",curve->X[i],curve->Y[i]);
  
  fclose(fp);
}


void WriteCurve2Gnuplot(Curve *curve,char *filename){
  FILE *fp;
  int i;
  
  fp = fopen(filename,"w");
  if (fp == NULL){
    fprintf(stderr,"Cannot open %s\n",filename);
    exit(-1);
  }
  fprintf(fp,"#%d\n",curve->n);
  for (i=0; i < curve->n; i++)
    fprintf(fp,"%f %f\n",curve->X[i],curve->Y[i]);
  
  fclose(fp);
}


Curve *CopyCurve(Curve *curve)
{
  Curve *curvec;

  curvec = CreateCurve(curve->n);
  memcpy(curvec->X,curve->X,curve->n*sizeof(double));
  memcpy(curvec->Y,curve->Y,curve->n*sizeof(double));
  return(curvec);
}