//#include "../include/bic.h"
#include "opencv2/imgproc/imgproc.hpp"
#include "opencv2/highgui/highgui.hpp"
#include <limits.h>

extern "C" {

#include "../include/bic.h"

}

using namespace cv;
using namespace std;

/// Apply Laplace function
Mat abs_dst, dst;
int kernel_size = 3;
int th = 20;


/*********************************************************
BIC
*********************************************************/

int *Quantize_colors(CImage *img, int color_dim) {
    unsigned long i;
    unsigned long r, g, b;
    unsigned long fator_g, fator_b;
    int *color, n;

    n = img->C[0]->nrows * img->C[0]->ncols;  

    color = (int *) calloc(n, sizeof(int));
    if(color==NULL){
        printf("\nOut of memory \n");
        exit(-1);
    }

    fator_g = color_dim;
    fator_b = fator_g*color_dim;

    for(i=0; i<n; i++) {
        r = color_dim*img->C[0]->val[i]/256;
        g = color_dim*img->C[1]->val[i]/256;
        b = color_dim*img->C[2]->val[i]/256;

        color[i] = (r + fator_g*g + fator_b*b);
    }

    return color;
}

/***************************************************************************/
unsigned char Compute_log(float value) {
    unsigned char result;

    value = 255. * value;
    if(value==0.)       result=0;
    else if(value<1.)   result=1;
    else if(value<2.)   result=2;
    else if(value<4.)   result=3;
    else if(value<8.)   result=4;
    else if(value<16.)  result=5;
    else if(value<32.)  result=6;
    else if(value<64.)  result=7;
    else if(value<128.) result=8;
    else                result=9;

    return result;
}

/**************************************************************************/
void Compress_histogram(unsigned char *ch, unsigned long *h, unsigned long max, int size) {
    int i;
    unsigned char v;

    for(i=0; i<size; i++) {
        v = Compute_log((float) h[i] / (float) max);
        ch[i] = (unsigned char)(48 + v);
    }
}


/***************************************************************************
  Este é o algoritmo BIC para classificar a cor entre interior/borda.
****************************************************************************/
void Compute_frequency_property(Image *img, Property *ppt) {
    unsigned long x, y, p, q;
    int i, border;
    AdjRel *A;
    Pixel v;

    A = Circular(1.0);

    for(y=0L; y<img->nrows; y++) {
        for(x=0L; x<img->ncols; x++) {
            p = x + img->tbrow[y];
            border=FALSE;
            for (i=1; i < A->n; i++) {
                v.x = x + A->dx[i];
                v.y = y + A->dy[i];
                if (ValidPixel(img,v.x,v.y)) {
                    q = v.x + img->tbrow[v.y];
                    if(ppt[p].color!=ppt[q].color) { 
                        border=TRUE;
                        break;
                    }
                }
            }
            if(border==FALSE) ppt[p].frequency=LOW;
            else ppt[p].frequency=HIGH;
        }
    }

    DestroyAdjRel(&A);
}

/****************************************************************************/
Property *Compute_pixels_properties(CImage *img) { 
    Property *p;
    int *color, i, n;

    n = img->C[0]->nrows * img->C[0]->ncols;  
    p = (Property *) calloc(n, sizeof(Property));

    if(p==NULL) {
        printf("\nOut of memory \n");
        exit(-1);
    }

    color = Quantize_colors(img, 4);
    for(i=0; i<n; i++) 
        p[i].color=color[i];
    Compute_frequency_property(img->C[0], p);

    free(color);
    return p;
}
/****************************************************************************/
VisualFeature *Compute_histograms(Property *p, int n) {
    VisualFeature *vf = (VisualFeature *) calloc(1,sizeof(VisualFeature));
    unsigned long i;

    for(i=0; i<DESC_SIZE; i++){
        vf->colorH[i] = 0;
        vf->lowH[i] = 0;
        vf->highH[i] = 0;
    }

    for(i=0; i<n; i++) {
        /*GCH*/
        vf->colorH[p[i].color]++;
        /*Frequency*/
        if(p[i].frequency==LOW) 
            vf->lowH[p[i].color]++;
        else 
            vf->highH[p[i].color]++;
    }

    return vf;
}

/**************************************************************************/

CompressedVisualFeature *Compress_histograms(VisualFeature *vf, int npixels) {
    CompressedVisualFeature *cvf =  (CompressedVisualFeature *) calloc(1,sizeof(CompressedVisualFeature));

    Compress_histogram(cvf->colorH, vf->colorH, npixels, DESC_SIZE);
    Compress_histogram(cvf->lowH, vf->lowH, npixels, DESC_SIZE);
    Compress_histogram(cvf->highH, vf->highH, npixels, DESC_SIZE);

    return cvf;
}

Property *Laplace(char *image, Property *p) {
    Mat src, src_gray;
    int ddepth = CV_16S;
    int iv, i, j, k, l, m, w, L, min, max;
    //int kernel_size = 3;
    int scale = 1;
    int delta = 0;
    int c, conta = 0;
    char* window_name = "Laplace Demo";
    Image *boundary;

    src = imread(image);
    /// Remove noise by blurring with a Gaussian filter
    GaussianBlur(src, src, Size(3,3), 0, 0, BORDER_DEFAULT);
    /// Convert the image to grayscale
    cvtColor(src, src_gray, CV_RGB2GRAY);
    /// Create window
    //namedWindow(window_name, CV_WINDOW_AUTOSIZE );
    Laplacian(src_gray, dst, ddepth, kernel_size, scale, delta, BORDER_DEFAULT);

    //imwrite("laplace.pgm", dst);
    //boundary = CreateImage(dst.cols, dst.rows);

    /*for (i=0; i < (dst.rows); i++) {
        for (j=0; j < (dst.cols); j++) {
            if (dst.at<short>(i,j) < 0) conta++;
            printf("%d ", dst.at<short>(i,j));
        }
    }*/
    //pega frequency do vetor p e muda de acordo com o valor do Laplace do pixel
    L = kernel_size;
    for (i=0; i < (dst.rows-L); i++) {
        for (j=0; j < (dst.cols-L); j++) {
            if (w < 0) { conta++;}
            min = INT_MAX; max = -min;
            for (k=0; k < L; k++) {
                for (l=0; l<L; l++) {
                    w=dst.at<short>(i+k,j+l);
                    if (w<min) min=w;
                    if (w>max) max=w;
                }
            }
            iv = j + i*dst.cols;
            if (min<-th && max>th) {
                //abs_dst.at<uchar>(i,j) = 255;
                p[iv].frequency = HIGH;
                //boundary->val[iv] = 255; 
            }
            else {
                //abs_dst.at<uchar>(i,j) = 0;
                p[iv].frequency = LOW; 
                //boundary->val[iv] = 0; 
            }
        }
    }

    //WriteImage(boundary, "boundary_laplace.pgm");
    //DestroyImage(&boundary);
    //printf ("Numero de valores negativos: %d\n", conta);
    //convertScaleAbs(dst, abs_dst);
    //imshow(window_name, abs_dst);
    //waitKey(0);
    return p;
}

/*************************************************************************/
Curve *BIC(CImage *img, char *image) {
    Curve *curve = CreateCurve(2*DESC_SIZE);
    CompressedVisualFeature *cvf;
    int i, j, pix, npixels;
    //int k, l, m, w, L, min, max;
    VisualFeature *vf;
    Property *p;

    npixels = img->C[0]->nrows * img->C[0]->ncols;

    //printf("Numero de pixels: %d\n", npixels);
    p = Compute_pixels_properties(img);
    //calcula o canny do img
    p = Laplace(image, p);

    //printf("\n");
    //for (i=0, pix=0; i < npixels; i++) if (p[i].frequency == HIGH) { printf("%d ", p[i].frequency); pix++; }
    //printf("\nNúmero de bordas: %d\n", pix);

    //Daqui pra baixo nao muda nada
    vf = Compute_histograms(p, npixels);
    cvf = Compress_histograms(vf, npixels);

    for (i=0; i<DESC_SIZE; i++) {
        curve->X[i] = i;
        curve->Y[i] = cvf->lowH[i];
        curve->X[i+DESC_SIZE] = i+DESC_SIZE;
        curve->Y[i+DESC_SIZE] = cvf->highH[i];
    }

    free(p);
    free(vf);
    free(cvf);
    return curve;
}