//#include "../include/bic.h"
#include "opencv2/imgproc/imgproc.hpp"
#include "opencv2/highgui/highgui.hpp"

extern "C" {

#include "../include/bic.h"

}

using namespace cv;
using namespace std;

Mat src, src_gray;
Mat dst, detected_edges;
Mat aux, exemplo;

int edgeThresh = 1;
int lowThreshold;
int const max_lowThreshold = 100;
int ratio = 3;
int kernel_size = 3;
//char* window_name = "Edge Map";

/*********************************************************
BIC
*********************************************************/

int *Quantize_colors(CImage *img, int color_dim){
  unsigned long i;
  unsigned long r, g, b;
  unsigned long fator_g, fator_b;
  int *color, n;
  
  n = img->C[0]->nrows * img->C[0]->ncols;  

  color = (int *) calloc(n, sizeof(int));
  if(color==NULL){
    printf("\nOut of memory \n");
    exit(-1);
  }
  
  fator_g = color_dim;
  fator_b = fator_g*color_dim;
  
  for(i=0; i<n; i++){
    r = color_dim*img->C[0]->val[i]/256;
    g = color_dim*img->C[1]->val[i]/256;
    b = color_dim*img->C[2]->val[i]/256;
    
    color[i] = (r + fator_g*g + fator_b*b);
  }
  return color;
}
/***************************************************************************/
unsigned char Compute_log(float value){
  unsigned char result;
  
  value = 255. * value;
  if(value==0.)       result=0;
  else if(value<1.)   result=1;
  else if(value<2.)   result=2;
  else if(value<4.)   result=3;
  else if(value<8.)   result=4;
  else if(value<16.)  result=5;
  else if(value<32.)  result=6;
  else if(value<64.)  result=7;
  else if(value<128.) result=8;
  else                result=9;
  
  return result;
}

/**************************************************************************/
void Compress_histogram(unsigned char *ch, unsigned long *h, 
      unsigned long max, int size){
  int i;
  unsigned char v;
  
  for(i=0; i<size; i++){
    v = Compute_log((float) h[i] / (float) max);
    ch[i] = (unsigned char)(48 + v);
  }
}


/***************************************************************************
  Este é o algoritmo BIC para classificar a cor entre interior/borda.
****************************************************************************/
void Compute_frequency_property(Image *img, Property *ppt){
  
  unsigned long x, y, p, q;
  int i, border;
  AdjRel *A;
  Pixel v;
  
  A = Circular(1.0);
  
  for(y=0L; y<img->nrows; y++){
    for(x=0L; x<img->ncols; x++){
      p = x + img->tbrow[y];
      border=FALSE;
      for (i=1; i < A->n; i++){
        v.x = x + A->dx[i];
        v.y = y + A->dy[i];
        if (ValidPixel(img,v.x,v.y)){
          q = v.x + img->tbrow[v.y];
          if(ppt[p].color!=ppt[q].color){ 
            border=TRUE;
            break;
          }
        }
      }
      if(border==FALSE) ppt[p].frequency=LOW;
      else ppt[p].frequency=HIGH;
    }
  }
  DestroyAdjRel(&A);
}

/****************************************************************************/
Property *Compute_pixels_properties(CImage *img)
{ 
  Property *p;
  int *color, i, n;
  
  n = img->C[0]->nrows * img->C[0]->ncols;  
  
  p = (Property *) calloc(n, sizeof(Property));
  if(p==NULL){
    printf("\nOut of memory \n");
    exit(-1);
  }
  
  color = Quantize_colors(img, 4);
  for(i=0; i<n; i++) 
    p[i].color=color[i];
  Compute_frequency_property(img->C[0], p);
  
  free(color);
  return p;
}
/****************************************************************************/
VisualFeature *Compute_histograms(Property *p, int n){
  VisualFeature *vf = (VisualFeature *) calloc(1,sizeof(VisualFeature));
  unsigned long i;
  
  for(i=0; i<DESC_SIZE; i++){
    vf->colorH[i] = 0;
    vf->lowH[i] = 0;
    vf->highH[i] = 0;
  }
  
  for(i=0; i<n; i++){
    /*GCH*/
    vf->colorH[p[i].color]++;
    
    /*Frequency*/
    if(p[i].frequency==LOW) 
      vf->lowH[p[i].color]++;
    else 
      vf->highH[p[i].color]++;
  }
  return vf;
}

/**************************************************************************/

CompressedVisualFeature *Compress_histograms(VisualFeature *vf, int npixels)
{
  CompressedVisualFeature *cvf =  (CompressedVisualFeature *) calloc(1,sizeof(CompressedVisualFeature));
  
  Compress_histogram(cvf->colorH, vf->colorH, npixels, DESC_SIZE);
  Compress_histogram(cvf->lowH, vf->lowH, npixels, DESC_SIZE);
  Compress_histogram(cvf->highH, vf->highH, npixels, DESC_SIZE);
  
  return cvf;
}

void CannyThreshold(int, void*) {
  float k = 0.23;
  /// Reduce noise with a kernel 3x3
  blur( src_gray, detected_edges, Size(3,3) );

  /// Canny detector
  Canny( detected_edges, detected_edges, lowThreshold, lowThreshold*ratio, kernel_size );

  //cout << "Dimensão detected_edges : " << detected_edges.size() << "\n";

  //imwrite("canny.pgm", detected_edges);
  /// Using Canny's output as a mask, we display our result
  dst = Scalar::all(0);
  src.copyTo( dst, detected_edges);

  /*cout << "Dimensão detected_edges: " << detected_edges.size() << "\n";
  cout << "detected_edges.type: " << detected_edges.type() << endl;

  cout << "Dimensão dst : " << dst.size() << "\n";
  cout << "dst.type: " << dst.type() << endl;

  for (size_t i=0; i < detected_edges.rows; i++) {
    cout << "\n";
    for (size_t j=0; j < detected_edges.cols; j++)
      printf("%d ", detected_edges.at<ushort>(i,j) );
  }
  cout << "\n";*/
}

/*************************************************************************/
Curve *BIC(CImage *img, char *image)
{
  Property *p;
  VisualFeature *vf;
  CompressedVisualFeature *cvf;
  int i, npixels;
  Curve *curve = CreateCurve(2*DESC_SIZE);
  
  npixels = img->C[0]->nrows * img->C[0]->ncols;
  
  p = Compute_pixels_properties(img);
  //calcula o canny do img
  src = imread(image);
  if( !src.data )
  { return -1; }
  /// Create a matrix of the same type and size as src (for dst)
  dst.create( src.size(), src.type() );
  cvtColor( src, src_gray, CV_BGR2GRAY );
  CannyThreshold(0, 0);

  //pega frequency do vetor p e muda de acordo com o valor do Canny do pixel
  //Daqui pra baixo nao muda nada
  vf = Compute_histograms(p, npixels);
  cvf = Compress_histograms(vf, npixels);
  
  for (i=0; i<DESC_SIZE; i++){
    curve->X[i] = i;
    curve->Y[i] = cvf->lowH[i];
    curve->X[i+DESC_SIZE] = i+DESC_SIZE;
    curve->Y[i+DESC_SIZE] = cvf->highH[i];
  }

  free(p);
  free(vf);
  free(cvf);
  return curve;
}